#include <fftw.h>
#include <math.h>
#include <time.h>
#include "gaussian.h"

int init_colors(int red[], int green[], int blue[], char *fname){
  int i;
  char s[256];
  for (i=0; i<14; i++) {
    sprintf (s, "red%d", i);
    red[i]=getint_ini(fname, s);
    sprintf (s, "green%d", i);
    green[i]=getint_ini(fname, s);
    sprintf (s, "blue%d", i);
    blue[i]=getint_ini(fname, s);
  }
  return 0;
}



int main(int argc, char **argv){
 int response;
 int n1, n2, wn1, wn2, nlevels, i;
 char covtype, fileformat, s[256];
 double theta[2], c[100];
 char *pf_num, *pf_clipped_num, *pf_ppm, *pf_clipped_ppm;
 int r[100],  g[100],  b[100];
 srand48(time(NULL));
 if (argc != 2) {
   printf ("Usage: %s ini_file_name\n", argv[0]);
   return -1;
 }
 init_colors (r, g, b, argv[1]);
 n1=getint_ini (argv[1], "n1");
 n2=getint_ini (argv[1], "n2");
 wn1=getint_ini  (argv[1], "wn1");
 wn2=getint_ini  (argv[1], "wn2");
 covtype=getstring_ini  (argv[1], "covtype")[0];
 if ((covtype != 's') &&
     (covtype != 'e') &&
     (covtype != 'r') &&
     (covtype != 'm')) {
       printf ("Unknown correlation '%c'\n", covtype);
       return 0;
 }
 theta[0]=getdouble_ini(argv[1], "theta1");
 theta[1]=getdouble_ini(argv[1], "theta2");
 pf_num = getstring_ini (argv[1], "numerical file for a Gaussian field");
 if (!*pf_num) {
  free (pf_num);
  pf_num=NULL;
 }
 pf_clipped_num  = getstring_ini (argv[1],
                 "numerical file for a discrete field");
 if (!*pf_clipped_num) {
  free (pf_clipped_num);
  pf_clipped_num=NULL;
 }
 pf_ppm = getstring_ini (argv[1],"ppm file for a Gaussian field");
 if (!*pf_ppm) {
  free (pf_ppm);
  pf_ppm=NULL;
 }
 pf_clipped_ppm = getstring_ini (argv[1], "ppm file for a discrete field");
 if (!*pf_clipped_ppm) {
  free (pf_clipped_ppm);
  pf_clipped_ppm=NULL;
 }
 fileformat=*getstring_ini (argv[1], "numeric file format");
 nlevels=getint_ini  (argv[1], "nlevels");
 for (i=0; i<nlevels; i++) {
    sprintf (s, "c%d", i+1);
    c[i]=getdouble_ini(argv[1], s);
 }
 response= write_field (n1, n2, wn1, wn2, theta, covtype,
                        pf_ppm, pf_num, pf_clipped_ppm, pf_clipped_num,
                        fileformat, nlevels, r, g, b, c, 0);
 switch(response) {
     case 0:  printf("Files created:\n");
              if (pf_num) printf ("  %s\n", pf_num);
              if (pf_clipped_num) printf ("  %s\n", pf_clipped_num);
              if (pf_ppm) printf ("  %s\n", pf_ppm);
              if (pf_clipped_ppm) printf ("  %s\n", pf_clipped_ppm);
              break;
     case -1: printf("Negative embedding.\n");
              break;
     case -2: printf("Out of memory.\n");
              break;
     case -3: printf("Can't compute Matern correlation.\n");
              break;
 }
 return 0;
}
