#include <sys/types.h>
#include <time.h>
#include <string.h>
#include <fftw.h>
#include "gaussian.h"



int init_colors(int red[], int green[], int blue[], char *fname){
  int i;
  char s[256];
  for (i=0; i<14; i++) {
    sprintf (s, "red%d", i);
    red[i]=getint_ini(fname, s);
    sprintf (s, "green%d", i);
    green[i]=getint_ini(fname, s);
    sprintf (s, "blue%d", i);
    blue[i]=getint_ini(fname, s);
  }
  return 0;
}



int main(int argc, char **argv){
 int i, n, n1, n2, d, r;
 int nlevels, gibbs_iterations;
 int red[100],  green[100],  blue[100];
 char file_format, *in_filename, s[256];
 char comment_c[256], comment_g[256], covtype;
 char *pf_num, *pf_clipped_num, *pf_ppm, *pf_clipped_ppm;
 double *a, *b, c[100], theta[2];
 double *enc_a, *enc_b;
 ghandle handle;
 if (argc != 2) {
   printf ("Usage: %s ini_file_name\n", argv[0]);
   return -1;
 }
 in_filename=getstring_ini (argv[1], "Discrete data file name");
 file_format=*getstring_ini (argv[1], "Discrete data file format");
 nlevels=getint_ini (argv[1], "nlevels");
 n1=getint_ini  (argv[1],"n1");
 n2=getint_ini  (argv[1], "n2");
 for (i=0; i<nlevels; i++) {
      sprintf (s, "threshold%d", i+1);
      c[i]=getdouble_ini (argv[1], s);
      sprintf (s, "code%d", i+1);
      red[i]=getint_ini (argv[1], s);
 }

 if (file_format=='p'){
   if (get_constraints_ppm (in_filename, &n1, &n2, nlevels, c, red, &a, &b)){
      printf ("Can't read file '%s'\n",in_filename);
      return -1;
   }
 }
 else{
   if (get_constraints_num (in_filename, file_format, n1, n2,
          nlevels, c, red, &a, &b)){
      printf ("Can't read file '%s'\n",in_filename);
      return -1;
   }
 }
 covtype=getstring_ini  (argv[1], "covtype")[0];
 switch (covtype){
            case 'r': sprintf(comment_g, "# Gaussian %dx%d r.f. \
with Rational Quadratic (%f, %f) correlation and mean 0.",
n1, n2, theta[0], theta[1]);
            break;
            case 's':sprintf(comment_g, "# Gaussian %dx%d r.f. \
with Spherical (%f) correlation and mean 0.",n1, n2, theta[0]);
            break;
            case 'e':sprintf(comment_g, "# Gaussian %dx%d r.f. \
with Exponential (%f, %f) correlation and mean 0.",
n1, n2, theta[0], theta[1]);
            break;
            case 'm':sprintf(comment_g, "# Gaussian %dx%d r.f. \
with Matern (%f, %f) correlation and mean 0.",
          n1, n2, theta[0], theta[1]);
  default: printf ("Unknown covariance type '%c'\n", covtype);
           return -1;
 }
 theta[0]=getdouble_ini (argv[1], "theta1");
 theta[1]=getdouble_ini (argv[1], "theta2");
 gibbs_iterations=getint_ini (argv[1], "gibbs_iterations");
 pf_num = getstring_ini (argv[1], "numerical file for a Gaussian field");
 if (!*pf_num) {
  free (pf_num);
  pf_num=NULL;
 }
 pf_clipped_num  = getstring_ini (argv[1],
                 "numerical file for a discrete field");
 if (!*pf_clipped_num) {
  free (pf_clipped_num);
  pf_clipped_num=NULL;
 }
 pf_ppm = getstring_ini (argv[1],"ppm file for a Gaussian field");
 if (!*pf_ppm) {
  free (pf_ppm);
  pf_ppm=NULL;
 }
 pf_clipped_ppm = getstring_ini (argv[1], "ppm file for a discrete field");
 if (!*pf_clipped_ppm) {
  free (pf_clipped_ppm);
  pf_clipped_ppm=NULL;
 }
 srand48(time(NULL));
 enc_a=(double *) malloc(4*n1*n2*sizeof(double));
 enc_b=(double *) malloc(4*n1*n2*sizeof(double));
 for (i=0; i<4*n1*n2; i ++) {
         enc_a[i]=-1000;
         enc_b[i]=1000;
 }
 d=0;
 for (n=0; n<n1; n++)
       for (r=0; r<n2; r++){
         enc_a[2*n*n2+r] = a[d];
         enc_b[2*n*n2+r] = b[d++];
       }
 if (!(handle = create_ghandle(n1, n2, FFTW_MEASURE))) {
    printf ("Not enough memory.\n");
    return -1;
 }
 sprintf(comment_c, "%s\n# Clipped at level(s) (", comment_g);
 for (i=0; i<nlevels-1; i++) {
    sprintf(s, "%f, ", c[i]);
    strcat(comment_c, s);
 }
 sprintf(s,"%f)", c[nlevels-1]);
 strcat(comment_c, s);
 if (pf_ppm || pf_clipped_ppm || pf_clipped_num || pf_num) {
  if (generate_w_given_x(handle, gibbs_iterations,
                               enc_a, enc_b, theta, covtype)) {
          memcpy(handle->k, handle->w, n1*n2*sizeof(FFTW_COMPLEX));
          z_from_w (handle);
          init_colors (red, green, blue, argv[1]);
          printf ("Created files:\n");
          if (pf_ppm) {
               write_ppm(handle->w, 0, pf_ppm, n1, n2);
               printf ("  %s\n",pf_ppm);
          }
          if (pf_clipped_num) {
              write_clipped_ppm (handle->w, 0, nlevels, c, red,
                  green, blue, pf_clipped_ppm, n1, n2);
              printf ("  %s\n",pf_clipped_ppm);
          }
          if (file_format=='p') file_format=3;
          if (pf_num){
             write_numeric (handle->w, 0, file_format,
                         pf_num, n1, n2, comment_g);
             printf ("  %s\n",pf_num);
          }
          if (pf_clipped_num){
            write_clipped_numeric (handle->w, 0, file_format,
                nlevels, c, pf_clipped_num, n1, n2, comment_c);
            printf ("  %s\n",pf_clipped_num);
          }
  }
  else printf ("Failure -- negative embedding.\n");
 } else printf ("No files to create.\n");
 free(enc_a);
 free(enc_b);
 free(a);
 free(b);
 return 0;
}
