#include <fftw.h>
#include <math.h>
#include "gaussian.h"
#include <strings.h>

int write_clipped_ppm (FFTW_COMPLEX *inpic,
                       int which,
                       int nlevels,
                       double *c,
                       int *r,
                       int *g,
                       int *b,
                       char *fname,
                       int rows,
                       int cols){
  /*
  take rows x cols field inpic, real or imaginary part depending on which,
  clip at nlevels c, write resulting image to fname using color
  r[i], g[i], b[i] for stuff less than c[i].
       e.g. three-color picture
            nlevels=2,
            c=(-0.43, 0.43)
            r=(153, 255, 51);
            g=(0,255,153);
            b=(0,0,255);
  */
    int  i, j, found;
    FILE *outf;
    outf = fopen (fname, "w");
    fprintf (outf, "P6\n%d %d\n255\n", cols, rows);
    if (which)
       for (i=0; i<rows*cols; i++)  {
           found=0;
           for (j=0; j<nlevels; j++)
             if (c_im(inpic[i]) < c[j]) {
                fprintf(outf, "%c%c%c", r[j], g[j], b[j]);
        found=1;
                break;
         }
       if (!found)
         fprintf(outf, "%c%c%c", r[nlevels], g[nlevels], b[nlevels]); /*high*/
       }
    else
       for (i=0; i<rows*cols; i++)  {
           found=0;
           for (j=0; j<nlevels; j++)
             if (c_re(inpic[i]) < c[j]) {
                fprintf(outf, "%c%c%c", r[j], g[j], b[j]);
        found=1;
                break;
         }
       if (!found)
         fprintf(outf, "%c%c%c", r[nlevels], g[nlevels], b[nlevels]); /*high*/
       }
    fclose(outf);
    return 0;
}

int write_clipped_numeric (FFTW_COMPLEX *inpic,
                           int which,
                           char format,
                           int nlevels,
                           double *c,
                           char *fname,
                           int rows,
                           int cols,
                           char *comment){
/* writes out clipped re(inpic) for which=0 or
              clipped im(inpic) for which=1;
The number of clipping thresholds is nlevels.
E.g. nlevels = 1 for binary data.
c[nlevels] holds the thresholds.
If format='3', the output format is
 x y z
 x y z
 x y z;
If format='Z', the output is Z values only.*/
    int  i, j, found, l;
    FILE *outf;
    outf = fopen (fname, "w");
    if (comment) fprintf(outf, "%s\n", comment);
    l=0;
    if (which)
       for (i=0; i<rows*cols; i++)  {
           found=0;
           for (j=0; j<nlevels; j++)
             if (c_im(inpic[i]) < c[j]) {
               found=1;
               switch (format){
                case '3': fprintf(outf, "%4d %4d     %d\n", i / cols, i % cols, j);
                          break;
                case 'Z': fprintf(outf, "%d ",  j);
                          l++;
                          if (l == 25) {
                            fprintf(outf, "\n");
                            l=0;
                          }
                          break;
                default: printf ("Unknown file format %c\n", format);
                           return -1;
               }
               if (found) break;
             }
             if (!found)
               switch (format){
                  case '3': fprintf(outf, "%4d %4d     %4d\n", i / cols, i % cols, nlevels);
                            break;
                  case 'Z': fprintf(outf, "%d ",  nlevels);
                            l++;
                            if (l == 25) {
                              fprintf(outf, "\n");
                              l=0;
                            }
                            break;
                  default: printf ("Unknown file format %c\n", format);
                           return -1;
               }
       }
    else
       for (i=0; i<rows*cols; i++)  {
           found=0;
           for (j=0; j<nlevels; j++)
             if (c_re(inpic[i]) < c[j]) {
               found=1;
               switch (format){
                case '3': fprintf(outf, "%4d %4d     %4d\n", i / cols, i % cols, j);
                          break;
                case 'Z': fprintf(outf, "%d ",  j);
                          l++;
                          if (l == 25) {
                            fprintf(outf, "\n");
                            l=0;
                          }
                          break;
                default: printf ("Unknown file format %c\n", format);
                           return -1;
               }
               if (found) break;
             }
             if (!found)
               switch (format){
                  case '3': fprintf(outf, "%4d %4d     %4d\n", i / cols, i % cols, nlevels);
                            break;
                  case 'Z': fprintf(outf, "%d ",  nlevels);
                            l++;
                            if (l == 25) {
                              fprintf(outf, "\n");
                              l=0;
                            }
                            break;
                  default: printf ("Unknown file format %c\n", format);
                           return -1;
               }
       }
    fclose(outf);
    return 0;
}


int write_numeric (FFTW_COMPLEX *inpic,
                   int which,
                   char format,
                   char *fname,
                   int rows,
                   int cols,
                   char *comment){
/* writes out re(inpic) for which=0 or im(inpic) for which=1;
if format='3', the output format is
 x y z
 x y z
 x y z;
if format='Z', the output is Z values only.*/
  FILE *outf;
  int i, j, lines, lastline;
  outf = fopen (fname, "w");
  if (comment) fprintf(outf, "%s\n", comment);
  switch (format){
    case '3':
     if (which==0)
       for (i=0; i<rows; i++)
         for (j=0; j<cols; j++)
             fprintf(outf, "%4d %4d    %12f\n", i, j, c_re(inpic[i*cols + j]));
     else
       for (i=0; i<rows; i++)
         for (j=0; j<cols; j++)
             fprintf(outf, "%4d %4d    %12f\n", i, j, c_im(inpic[i*cols + j]));
     break;
    case 'Z':
     lines= (rows * cols) / 7;
     lastline = (rows * cols) % 7;
     if (which==0) {
           for (i=0; i < lines; i++){
             for (j=0; j<7;  j++)
                 fprintf(outf, "%12f", c_re(inpic[i*7 + j]));
             fprintf(outf, "\n");
           }
           for (j=0; j < lastline; j++)
             fprintf(outf, "%12f", c_re(inpic[lines*7 + j]));
           fprintf(outf, "\n");
           break;
     }
     else {
           for (i=0; i < lines; i++){
             for (j=0; j<7;  j++)
                 fprintf(outf, "%12f", c_im(inpic[i*7 + j]));
             fprintf(outf, "\n");
           }
           for (j=0; j < lastline; j++)
             fprintf(outf, "%12f", c_im(inpic[lines*7 + j]));
           fprintf(outf, "\n");
           break;
     }
   default:
     printf("Unknown file format: %c.\n", format);
     fclose(outf);
     return -1;
     break;
  }
  fclose(outf);
  return 0;
}


int read_numeric (FFTW_COMPLEX *m,
                  int which,
                  char format,
                  char *fname,
                  int rows,
                  int cols){
      FILE *inp;
      char s[256];
      int i, j;
      /*-----count lines starting with # ----------*/
      if (!(inp=fopen(fname, "r"))) return -1;
      i=0;
      fgets(s, 256, inp);
      while ((s[0]=='#') && (!feof (inp))) {
        i++;
        fgets(s, 256, inp);
      }
      fclose(inp);
      /*-----skip lines starting with # ----------*/
      if (!(inp=fopen(fname, "r"))) return -1;
      for (j=0; j<i; j++) fgets(s, 256, inp);
      /*-----read file starting with # ----------*/
      if (which) {
           switch (format){
             case '3': for (i=0; i<rows*cols; i++){
                           fgets(s, 256, inp);
                           sscanf(s, "%*f%*f%lf", &c_im(m[i]));
                       }
                       break;
             case 'Z': for (i=0; i<rows*cols; i++){
                           fscanf(inp, "%lf", &c_im(m[i]));
                       }
                       break;
             default: fclose(inp);
                      return -1;
           }
      }
      else {
           switch (format){
             case '3': for (i=0; i<rows*cols; i++){
                           fgets(s, 256, inp);
                           sscanf(s, "%*f%*f%lf", &c_re(m[i]));
                       }
                       break;
             case 'Z': for (i=0; i<rows*cols; i++){
                           fscanf(inp, "%lf", &c_re(m[i]));
                       }
                       break;
             default:  fclose(inp);
                       return -1;
           }
      }
      fclose(inp);
      return 0;
}

int write_ppm(FFTW_COMPLEX *m, int which, char *fname, int rows, int cols){
  /* write image stored in m to color ppm file
     which=0 means use real part of m
     which=1 means use imaginary part of m
     image size is rows \times cols. */
  FILE *outf;
  int i, pixel;
  double max, min, seg_length;
  if (which==0) {
     max=c_re(m[0]);
     min=c_re(m[0]);
     for (i=0; i < rows*cols; i++) {
       if (c_re(m[i])>max) max=c_re(m[i]);
       if (c_re(m[i])<min) min=c_re(m[i]);
     }
  } else {
           max=c_im(m[0]);
           min=c_im(m[0]);
           for (i=0; i < rows*cols; i++) {
              if (c_im(m[i])>max) max=c_im(m[i]);
              if (c_im(m[i])<min) min=c_im(m[i]);
       }
    }
  seg_length=(max - min)/255;
  outf = fopen (fname, "w");
  fprintf (outf, "P6\n%d %d\n255\n", cols, rows);
  if (which==0) {
    for (i=0; i<rows*cols; i++) {
           pixel = (c_re(m[i]) - min) / seg_length;
           fprintf(outf, "%c%c%c", pixel,pixel,pixel);
    }
  } else {
      for (i=0; i<rows*cols; i++){
           pixel = (c_im(m[i]) - min) / seg_length;
           fprintf(outf, "%c%c%c", pixel,pixel,pixel);
    }
    }
  fclose(outf);
  return 0;
}

int get_constraints_ppm (char *fname,
                         int *rows,
                         int *cols,
                         int nlevels,
                         double *c,
                         int *r,
                         double **a,
                         double **b){
  /* Get image dimensions rows x cols and constraints (observed data)
     from ppm file *fname and levels. The nlevels, levels c, and red signals r of each level
     are given. Allocate memory and populate a, b so that a .< z .< b.
     e.g. c=(-0.43, 0.43) nlevels=2, three-color picture
          r=(153, 255, 51); but 51 here is not important and will
          not be used.
  */
  FILE *picfile;
  int i, l;
  unsigned char j, k, level;
  char dummy[256];
  picfile = fopen (fname, "r");
  if (picfile==NULL) return -1;
  else {
     dummy[0]='#';
     while (dummy[0] == '#') fgets (dummy, 256, picfile);
     /*got p2; insert filetype check here */
     dummy[0]='#';
     while (dummy[0] == '#') fgets (dummy, 256, picfile);
     /*got rows cols */
     sscanf (dummy, "%d %d", rows, cols);
     dummy[0]='#';
     while (dummy[0] == '#') fgets (dummy, 256, picfile);
     /*got maxlevel*/
     *a = (double *) malloc ((*rows)*(*cols)*sizeof(double));
     *b = (double *) malloc ((*rows)*(*cols)*sizeof(double));
     for (i=0; i < (*rows)*(*cols); i++) {
      fscanf (picfile, "%c%c%c", &level, &k, &j);
      if ((int)level == r[0]) {
         (*a)[i]=-1000;
         (*b)[i]=c[0];
      } else {
         (*a)[i]=c[nlevels-1];
         (*b)[i]=1000;
        }
      for (l=1; l < nlevels; l++)
        if ((int)level == r[l]) {
            (*a)[i]=c[l-1];
            (*b)[i]=c[l];
        }
     }
  return 0;
  }
}

int get_constraints_num (char *fname,
                         char file_format,
                         int rows,
                         int cols,
                         int nlevels,
                         double *c,
                         int *codes,
                         double **a,
                         double **b){
  /* For rows x cols image,  get constraints (observed data)
     from numeric file *fname and levels.
     The nlevels, levels c, and codes of each level
     are given. Allocate memory and populate a, b so that a .< z .< b.
     e.g. for three color image, c=(-0.43, 0.43), nlevels=2,
          codes=(0, 1, 2); but 2 here is not important and will
          not be used.
  */
  FFTW_COMPLEX *field;
  int i, l;
  *a = (double *) malloc (rows*cols*sizeof(double));
  *b = (double *) malloc (rows*cols*sizeof(double));
  field = (FFTW_COMPLEX *) malloc(rows*cols*sizeof (FFTW_COMPLEX));
  read_numeric (field, 0, file_format, fname, rows, cols);
  for (i=0; i<rows*cols; i++) {
        if ((int)c_re(field[i]) == codes[0]) {
               (*a)[i]=-1000;
               (*b)[i]=c[0];
            } else {
               (*a)[i]=c[nlevels-1];
               (*b)[i]=1000;
              }
            for (l=1; l < nlevels; l++)
              if ((int)c_re(field[i]) == codes[l]) {
                  (*a)[i]=c[l-1];
                  (*b)[i]=c[l];
              }
  }
  free(field);
  return 0;
}

int getint_ini (char *fname, char *lhs){
/*
 Get integer value from the string in file named fname of the form
 lhs=2 or lhs:2
*/
  FILE *inp;
  char s[256], *ptr;
  int result;
  if (inp=fopen(fname, "r")){
    while (!feof(inp)){
       fgets(s, 256, inp);
       if (!strncasecmp(lhs, s, strlen(lhs))) break;
       else s[0]='\0';
    }
    fclose(inp);
    if (s[0]){
         ptr = (char *)strstr(s,":");
         if (!ptr) ptr=(char *)strstr(s,"=");
         if (!ptr) {
          printf ("Can't parse line '%s' in ini file '%s'\n",
            lhs, fname);
          exit(-1);
         }
         if (sscanf(ptr+1, "%d", &result)) return result;
         printf ("Can't parse line '%s' in ini file '%s'\n",
            lhs, fname);
         exit(-1);
    }
    printf ("Can't find string '%s' in ini file '%s'.\n", lhs, fname);
    exit (-1);
  }
  printf ("File not found: '%s'\n", fname);
  exit (-1);
}

double getdouble_ini (char *fname, char *lhs){
/*
 Get double value from the string in file named fname of the form
 lhs=2 or lhs:2
*/
  FILE *inp;
  char s[256], *ptr;
  double result;
  if (inp=fopen(fname, "r")){
    while (!feof(inp)){
       fgets(s, 256, inp);
       if (!strncasecmp(lhs, s, strlen(lhs))) break;
       else s[0]='\0';
    }
    fclose(inp);
    if (s[0]){
         ptr= (char *)strstr(s,":");
         if (!ptr) ptr=(char *)strstr(s,"=");
         if (!ptr) {
            printf ("Can't parse line '%s' in ini file '%s'\n",
               lhs, fname);
            exit(-1);
         }
         if (sscanf(ptr+1, "%lf", &result)) return result;
         printf ("Can't parse line '%s' in ini file '%s'\n",
            lhs, fname);
         exit(-1);
    }
    printf ("Can't find string '%s' in ini file '%s'.\n", lhs, fname);
    exit (-1);
  }
  printf ("Ini file not found: '%s'\n", fname);
  exit (-1);
}

char *getstring_ini (char *fname, char *lhs){
/*
 Get string value from the string in file named fname of the form
 lhs=2 or lhs:2
*/
  FILE *inp;
  char s[256], *ptr, *result;
  result = (char *) malloc(256);
  if (inp=fopen(fname, "r")){
    while (!feof(inp)){
       fgets(s, 256, inp);
       if (!strncasecmp(lhs, s, strlen(lhs))) break;
       else s[0]='\0';
    }
    fclose(inp);
    if (s[0]){
         /* hack for IBM PC strings */
         if (s[strlen(s)-2]==13) s[strlen(s)-2]='\0';
         else s[strlen(s)-1]='\0';
         ptr= (char *)strstr(s,":");
         if (!ptr) ptr=(char *)strstr(s,"=");
         if (!ptr) {
             printf ("Can't parse line '%s' in ini file '%s'\n",
               lhs, fname);
             exit(-1);
         }
         while (*(ptr+1)==' ') ptr++;
         strcpy (result, ptr+1);
         return result;
    }
    printf ("Can't find string '%s' in ini file '%s'.\n", lhs, fname);
    exit (-1);
  }
  printf ("Ini file not found: '%s'\n", fname);
  exit (-1);
}
