#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "gaussian.h"

/* generate unif [rmin,rmax] */
double runif (double rmin, double rmax){
   return drand48() * (rmax-rmin) + rmin;
}

/* generate a discrete RV taking n integer values v[]
                               with probabilities p[] */
int discrv (int v[], double p[], int n){
  double z, psum;
  int i;
  if (n==1) return v[0];
  z = (double)rand() / RAND_MAX;
  psum = p[0];
  for (i=0; i<n; i++) {
     if (z < psum) return v[i];
     psum += p[i+1];
  }
  return 0;
}


/* generate standard normal */
double n01 () {
  double u, v, w, r;
  int done;
  done = 0;
  while (!done){
    u = runif (-1, 1);
    v = runif (-1, 1);
    r = u*u + v*v;
    if (r <= 1) done = 1;
  }
  w = sqrt (-2*log(r)/r);
  return u*w;
}


double normal (double mu, double sigma2) {
  return n01() * sqrt(sigma2) + mu;
}


double trunc_normal (double mu, double sigma2, double a, double b){
  /* return Z ~ N(mu, sigma2) such that a<Z<b */
  double l, r, u01, arg;
  l=Phi((a-mu)/sqrt(sigma2));
  r=Phi((b-mu)/sqrt(sigma2));
  u01=runif(0,1);
  arg = l+(r-l)*u01;
  return mu+sqrt(sigma2)*PhiInv(arg);
}
